/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */
package org.dive4elements.river.artifacts.uinfo.salix;

import java.util.List;

import org.dive4elements.artifactdatabase.state.Facet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.ChartArtifact;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.model.Calculation;
import org.dive4elements.river.artifacts.model.CalculationResult;
import org.dive4elements.river.artifacts.model.DataFacet;
import org.dive4elements.river.artifacts.model.EmptyFacet;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.artifacts.model.ReportFacet;
import org.dive4elements.river.artifacts.resources.Resources;
import org.dive4elements.river.artifacts.states.DefaultState;
import org.dive4elements.river.artifacts.uinfo.UINFOArtifact;

/**
 * @author Domenico Nardi Tironi
 */
public class SalixLineState extends DefaultState {

    private static final long serialVersionUID = 1L;

    /**
     * From this state can only be continued trivially.
     */
    @Override
    protected String getUIProvider() {
        return "continue";
    }

    @Override
    public Object computeFeed(final D4EArtifact artifact, final String hash, final CallContext context, final List<Facet> facets, final Object old) {
        // FIXME: why is this necessary?
        if (artifact instanceof ChartArtifact) {
            facets.add(new EmptyFacet());
            return null;
        }

        return compute((UINFOArtifact) artifact, context, hash, facets, old);
    }

    @Override
    public Object computeAdvance(final D4EArtifact artifact, final String hash, final CallContext context, final List<Facet> facets, final Object old) {
        if (artifact instanceof ChartArtifact) {
            facets.add(new EmptyFacet());
            return null;
        }
        return compute((UINFOArtifact) artifact, context, hash, facets, old);
    }

    /**
     * Compute result or returned object from cache, create facets.
     *
     * @param old
     *            Object that was cached.
     */
    private Object compute(final UINFOArtifact uinfo, final CallContext context, final String hash, final List<Facet> facets, final Object old) {

        final CalculationResult res = doCompute(uinfo, context, old);

        if (facets == null)
            return res;

        final SalixLineCalculationResults results = (SalixLineCalculationResults) res.getData();

        final List<SalixLineCalculationNoScenarioResult> resultList = results.getResults();

        if (!resultList.isEmpty()) {
            facets.add(SalixLineProcessor.createSalixLineFacet(context, hash, this.id, resultList.get(0), 0, 0));
            facets.add(SalixLineProcessor.createSalixMnwMwFacet(context, hash, this.id, resultList.get(0), 0, 0));
            if (resultList.get(0) instanceof SalixLineCalculationRegionalResult) {
                final SalixLineCalculationRegionalResult result = (SalixLineCalculationRegionalResult) resultList.get(0);
                for (int i = 0; i <= result.getScenarioCount() - 1; i++) {
                    final String sublabel = Resources.getMsg(context.getMeta(), "uinfo_salix_scenario_deltaw", "uinfo_salix_scenario_deltaw",
                            result.getScenarioLabel(i));
                    facets.add(SalixLineProcessor.createSalixScenarioFacet(context, hash, this.id, result, i, 0, sublabel));
                }
            }
            else if (resultList.get(0) instanceof SalixLineCalculationSupraRegionalResult) {
                final String sublabel = Resources.getMsg(context.getMeta(), "uinfo_salix_scenario_supraregional");
                facets.add(SalixLineProcessor.createSalixScenarioFacet(context, hash, this.id, resultList.get(0), 0, 0, sublabel));
            }

            else if (resultList.get(0) instanceof SalixLineCalculationHistoricalResult) {
                final String sublabel = Resources.getMsg(context.getMeta(), "uinfo_salix_scenario_historical");
                facets.add(SalixLineProcessor.createSalixScenarioFacet(context, hash, this.id, resultList.get(0), 0, 0, sublabel));
            }

            final Facet csv = new DataFacet(FacetTypes.CSV, "CSV data", ComputeType.ADVANCE, hash, this.id);
            final Facet pdf = new DataFacet(FacetTypes.PDF, "PDF data", ComputeType.ADVANCE, hash, this.id);

            facets.add(csv);
            facets.add(pdf);
        }

        final Calculation report = res.getReport();

        if (report.hasProblems())
            facets.add(new ReportFacet(ComputeType.ADVANCE, hash, this.id));

        return res;
    }

    private CalculationResult doCompute(final UINFOArtifact uinfo, final CallContext context, final Object old) {
        if (old instanceof CalculationResult)
            return (CalculationResult) old;

        return new SalixLineCalculation(context).calculate(uinfo);
    }
}